"""
w2lapp.core.py: some core functions used throughout web2ldap

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: core.py,v 1.62 2002/12/29 22:48:37 michael Exp $
"""

import sys,os,signal,types,time,traceback,\
       ldap,pyweblib.forms,pyweblib.session,ldapsession,msbase, \
       w2lapp.cnf,w2lapp.schema

from pyweblib.forms import escapeHTML

def str2unicode(s,charset):
  if type(s) is types.StringType:
    try:
      return unicode(s,charset)
    except UnicodeError:
      return unicode(s,'iso-8859-1')
  else:
    return s

def utf2display(dispcharset,value,escape_entities=1):
  if type(value)==types.UnicodeType:
    unicode_value = value
  elif type(value) is types.StringType:
    if value.startswith('{T.61}'):
      unicode_value = unicode(value[6:],'iso-8859-1')
    else:
      try:
        unicode_value = unicode(value,'utf-8')
      except UnicodeError:
        unicode_value = unicode(value,'iso-8859-1')
  elif value is None:
    return ''
  elif type(value) is types.ListType:
    return [
      utf2display(dispcharset,i,escape_entities)
      for i in value
    ]
  else:
    raise TypeError, "expected string or list"
  if escape_entities:
    return escapeHTML(unicode_value.encode(dispcharset,'replace'))
  else:
    return unicode_value.encode(dispcharset,'replace')

########################################################################
# Initialize some constants
########################################################################

ldap._trace_level=w2lapp.cnf.misc.ldap_trace_level
ldap.set_option(ldap.OPT_RESTART,0)
ldap.set_option(ldap.OPT_DEREF,0)
ldap.set_option(ldap.OPT_REFERRALS,0)
#ldap.set_option(ldap.OPT_TIMEOUT,60.0)
#ldap.set_option(ldap.OPT_NETWORK_TIMEOUT,60.0)

startUpTime = time.time()


class Session(pyweblib.session.WebSession):

  def deleteSession(self,sid):
    try:
      ls_local = self.sessiondict[sid][1]
    except KeyError:
      pass
    else:
      if isinstance(ls_local,ldapsession.LDAPSession):
        ls_local.unbind()
    pyweblib.session.WebSession.deleteSession(self,sid)


# Initialize web session object
session = Session(
  expireDeactivate=w2lapp.cnf.misc.session_relogin,
  expireRemove=w2lapp.cnf.misc.session_remove,
  crossCheckVars = w2lapp.cnf.misc.session_checkvars,
  maxSessionCount = w2lapp.cnf.misc.session_limit,
)

class CleanUpThread(pyweblib.session.CleanUpThread):
  """
  Thread class for clean-up thread
  
  Mainly it overrides pyweblib.session.CleanUpThread.run()
  to call ldapSession.unbind().
  """

  def run(self):
    """Thread function for cleaning up session database"""
    while not self._stop_event.isSet():
      try:
        current_time = time.time()
        sessiondict_keys = [
          id
          for id in self._sessionInstance.sessiondict.keys()
          if not id.startswith('__')
        ]
        for session_id in sessiondict_keys:
          try:
            session_timestamp,ldapSession = self._sessionInstance.sessiondict[session_id]
          except KeyError:
            # Avoid race condition. The session might have been
            # deleted in the meantime. But make sure everything is deleted.
            self._sessionInstance.deleteSession(session_id)
          else:
            # Check expiration time
            if session_timestamp+self._sessionInstance.expireRemove<current_time:
              # Remove expired session
              self._sessionInstance.deleteSession(session_id)
              self._removed+=1
            elif (session_timestamp+self._sessionInstance.expireDeactivate<current_time):
              if ldapSession=='_created_' or not hasattr(ldapSession,'uri'):
                # Completely hanging sessions are removed immediately
                self._sessionInstance.deleteSession(session_id)
                self._removed+=1
              else:
                # Disconnect from LDAP server
                ldapSession.unbind()
      except:
        # Catch all exceptions to avoid thread being killed.
        if __debug__:
          traceback.print_exc()
        pass

      # Sleeping until next turn
      self._stop_event.wait(self._interval)

    return # CleanUpThread.run()


# Mapping from sndhdr.what() result to MIME-type
sndhdr_mimetype = {
  'au':'audio/basic',
  'wav':'audio/wav',
  'aiff':'audio/x-aiff',
  'aifc':'audio/x-aiff',
}
# Mapping from imghdr.what() result to MIME-type
imghdr_mimetype = {
  'bmp':'image/bmp',
  'gif':'image/gif',
  'jpeg':'image/jpeg',
  'pbm':'image/x-portable-bitmap',
  'pgm':'image/x-portable-graymap',
  'png':'image/png',
  'ppm':'image/x-portable-pixmap',
  'rast':'image/x-cmu-raster',
  'rgb':'image/x-rgb',
  'tiff':'image/tiff',
  'xbm':'image/x-xbitmap',
}

ldap_binaryattr = ldap.cidict.cidict(w2lapp.cnf.misc.ldap_binaryattr)
ldap_binaryattrkeys = ldap_binaryattr.keys()

class ErrorExit:
  """Base class for web2ldap application exceptions"""
  def __init__(self,Msg):
    self.Msg = Msg

def log_exception(errf,ls,env):
  """
  Write an exception with environment vars, LDAP connection data
  and Python traceback to errf file object.
  """
  # Get exception instance and traceback info
  exc_obj,exc_value,exc_traceback = sys.exc_info()
  # Signals are raised again to trigger handling in main process
  if isinstance(exc_value,KeyboardInterrupt):
    raise
  if isinstance(exc_value,IOError) and (exc_value.errno==32):
    pass
  else:
    logentry = ['%s\nUnhandled error at %s' % (
        '-'*60,
        time.strftime(
          '%Y-%m-%dT%H:%M:%SZ',time.gmtime(time.time())
        ),
      )]
    logentry.append('LDAPSession instance: %s' % repr(ls))
    # Log all environment vars
    env_vars = env.keys();env_vars.sort()
    for v in env_vars:
      logentry.append('%s: "%s"' % (v,env[v]))
    logentry.append(''.join(traceback.format_exception(exc_obj,exc_value,exc_traceback,20)))
    # Write the log entry to errf file object
    errf.write(os.linesep.join(logentry))
  # Avoid memory leaks
  exc_obj=None;exc_value=None;exc_traceback=None
  del exc_obj;del exc_value;del exc_traceback
