"""
w2lapp.read.py: Read existing entry and output as HTML or vCard

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: read.py,v 1.81 2002/11/20 23:51:46 michael Exp $
"""

import msbase,pyweblib.forms,ldap.schema,ldaputil.base,pyweblib.httphelper, \
       w2lapp.core,w2lapp.cnf,w2lapp.gui,w2lapp.read,w2lapp.viewer
from msbase import listremove,intersection,union


class DisplayEntry(ldap.schema.Entry):

  def __init__(self,sid,form,ls,dn,schema,entry):
    ldap.schema.Entry.__init__(self,schema,None,entry)
    self.sid = sid
    self.form = form
    self.ls = ls
    self.dn = dn
    self.sep = '<br>'

  def __getitem__(self,nameoroid):
    oid = self._at2key(nameoroid)[0]
    try:
      values = ldap.schema.Entry.__getitem__(self,nameoroid)
    except KeyError:
      return '&nbsp;'
    else:
      if oid=='2.5.4.0':
        # objectClass attribute is displayed with different function
        result = [
          w2lapp.gui.SchemaElementName(
            self.sid,self.form,self.dn,self._s,v,ldap.schema.ObjectClass
          )
          for v in values
        ]
      else:
        result = [
          w2lapp.gui.DataStr(
            self.sid,self.form,self.ls,self.dn,self._s,
            nameoroid,values[i],
            valueindex=i,
            commandbutton=1
          )
          for i in range(len(values))
        ]
      return '<br>'.join(result)


def getOperationAttrsTemplate(ls,accept_language):
  template_pathname = w2lapp.cnf.GetParam(ls,'read_operationalattrstemplate')
  if not template_pathname:
    return ''
  template_filename = w2lapp.gui.GetVariantFilename(
    template_pathname,accept_language
  )
  return open(template_filename,'r').read()


def PrintAttrList(sid,outf,form,ls,dn,sub_schema,entry,attrs,Comment=''):
  show_attrs = [
    a
    for a in attrs
    if entry.has_key(a)
  ]
  if not show_attrs:
    return
  show_attrs.sort()
  if Comment:
    outf.write('<h2>%s</h2>\n<dl class="ReadAttrList">\n' % (Comment))
  for attr_type_name in show_attrs:
    attr_type_str = w2lapp.gui.SchemaElementName(sid,form,dn,sub_schema,attr_type_name,ldap.schema.AttributeType)
    if w2lapp.schema.no_humanreadable_attr(sub_schema,attr_type_name):
      dt_list = [attr_type_str]
      dt_list.append(w2lapp.gui.W2L_Anchor(
        form,'delete','Delete',sid,
        [('dn',dn),('delete_attr',attr_type_name)]
      ))
      dt_list.append(w2lapp.gui.W2L_Anchor(
	form,'read','Save to disk',sid,
        [
          ('dn',dn),
          ('read_attr',attr_type_name),
          ('read_attrmode','load'),
          ('read_attrmimetype','application/octet-stream'),
          ('read_attrindex','0'),
        ],
      ))
      dt_str = '<table>\n<tr>\n<td>%s</td>\n</tr>\n</table>' % (
        '</td>\n<td>|</td>\n<td>'.join(dt_list)
      )
    else:
      dt_str = attr_type_str
    outf.write('<dt>\n%s\n<dt>\n<dd>%s</dd>\n' % (
      dt_str,entry[attr_type_name]
    ))
  outf.write('</dl>\n')


def w2l_Read(
  sid,outf,command,form,ls,dn,
  wanted_attrs,
  read_attrmode = 'view',
  read_attrmimetype = None,
):

  sub_schema = ls.retrieveSubSchema(dn,w2lapp.cnf.GetParam(ls,'_schema',None))

  read_output = form.getInputValue('read_output',['template'])[0]

  read_nocache = int(form.getInputValue('read_nocache',['0'])[0])
  if read_nocache:
    ls.uncacheEntry(dn)

  # Specific attributes requested?
  wanted_attrs = [
    a.encode('ascii')
    for a in form.getInputValue('read_attr',wanted_attrs)
  ]

  # Determine how to get all attributes including the operational attributes

  operational_attrs_template = getOperationAttrsTemplate(
    ls,form.accept_language
  )

  # Read the entry's data
  if wanted_attrs:
    # Read only wanted attributes
    search_result = ls.readEntry(dn,wanted_attrs)
  else:
    # Read all user and operational attributes
    search_result = ls.readEntry(dn,{0:None,1:['*','+']}[ls.supportsAllOpAttr])

  if not search_result:
    raise w2lapp.core.ErrorExit('Empty search result.')

  entry = ldap.schema.Entry(sub_schema,dn,search_result[0][1])

  requested_attrs = msbase.union(
    msbase.GrabKeys(operational_attrs_template)(),
    w2lapp.cnf.GetParam(ls,'requested_attrs',[]),
    ignorecase=1
  )
  if not wanted_attrs and requested_attrs:
    try:
      search_result = ls.readEntry(dn,requested_attrs)
    except ldap.NO_SUCH_ATTRIBUTE:
      # Catch and ignore complaints of server about not knowing attribute
      pass
    else:
      if search_result:
        entry.update(search_result[0][1])

  display_entry = DisplayEntry(sid,form,ls,dn,sub_schema,entry)

  # Save session into database mainly for storing LDAPSession cache
  w2lapp.core.session.storeSession(sid,ls)

  if len(wanted_attrs)==1:

    # Display a single binary attribute either with a registered
    # viewer or just by sending the data blob with appropriate MIME-type

    attr_type = wanted_attrs[0]
    if not entry.has_key(attr_type):
      raise w2lapp.core.ErrorExit('Attribute not in entry.')

    # Send a single binary attribute with appropriate MIME-type
    read_attrindex = int(form.getInputValue('read_attrindex',['0'])[0])
    # Determine if user wants to view or download the binary attribute value
    read_attrmode = form.getInputValue('read_attrmode',[read_attrmode])[0]
    attr_type_oid = sub_schema.getoid(ldap.schema.AttributeType,attr_type)
    syntax_se = w2lapp.schema.syntaxes.syntax_registry.syntaxClass(sub_schema,attr_type)
    if (read_attrmode=='view') and \
       hasattr(syntax_se,'oid') and \
       (w2lapp.viewer.viewer_func.has_key(syntax_se.oid)):
      # Nice displaying of binary attribute with viewer class
      w2lapp.gui.TopSection(
        sid,outf,form,ls,dn,
        w2lapp.core.utf2display(form.accept_charset,dn),
        w2lapp.gui.MainMenu(sid,form,ls,dn),
        context_menu_list=w2lapp.gui.ContextMenuSingleEntry(sid,form,ls,dn)
      )
      outf.write('<div id=MessageDiv>\n')
      w2lapp.viewer.viewer_func[syntax_se.oid](
	sid,outf,command,form,dn,attr_type,entry,read_attrindex
      )
      outf.write('</div>\n')
      w2lapp.gui.PrintFooter(outf,form)
    elif (read_attrmode=='load'):
      # Raw download with (hopefully) appropriate MIME-type
      read_attrmimetype = form.getInputValue('read_attrmimetype',['application/octet-stream'])[0]
      w2lapp.viewer.DisplayBinaryAttribute(
        sid,outf,command,form,dn,attr_type,entry,
        index=read_attrindex,mimetype=read_attrmimetype
      )
    else:
      raise w2lapp.core.ErrorExit('Illegal value for form parameter read_attrmode.')
    return

  if read_output in ['table','template']:

    # Display the whole entry

    w2lapp.gui.TopSection(
      sid,outf,form,ls,dn,
      w2lapp.core.utf2display(form.accept_charset,dn),
      w2lapp.gui.MainMenu(sid,form,ls,dn),
      context_menu_list=w2lapp.gui.ContextMenuSingleEntry(sid,form,ls,dn)
    )

    displayed_attrs = {}

    # List of already displayed attributes

    outf.write('<div id=MessageDiv>\n')

    if read_output=='template':

      ############################################################
      # Template display
      ############################################################

      # Determine relevant HTML templates
      read_template_dict = ldap.cidict.cidict(w2lapp.cnf.GetParam(ls,'read_template',{}))
      template_oc = intersection(
        [ s.lower() for s in entry.get('objectClass',[]) ],
        [ s.lower() for s in read_template_dict.keys() ],
        ignorecase=1
      )
      # Templates defined => display the entry with the help of the template
      used_templates = []
      for oc in template_oc:
	read_template_filename = w2lapp.gui.GetVariantFilename(read_template_dict[oc],form.accept_language)
        if not read_template_filename in used_templates:
          used_templates.append(read_template_filename)
          if read_template_filename:
	    try:
  	      template_str = open(read_template_filename,'r').read()
	    except IOError:
	      outf.write('<p>I/O error during reading template file for object class <strong>%s</strong>.</p>' % oc)
            else:
  	      outf.write(template_str % display_entry)
              for attr_type_name in msbase.GrabKeys(template_str)():
                displayed_attrs[sub_schema.getoid(ldap.schema.AttributeType,attr_type_name)] = None
	  else:
            outf.write('<p>Template file for object class <strong>%s</strong> not found.</p>' % oc)

    ##############################################################
    # Raw display
    ##############################################################

    required_attrs_dict,allowed_attrs_dict = entry.attribute_types(
      strict=0,raise_keyerror=0
    )

    required_attrs = []
    allowed_attrs = []
    nomatching_attrs = []
    for a in entry.keys():
      a_oid = sub_schema.getoid(ldap.schema.AttributeType,a)
      if displayed_attrs.has_key(a_oid):
        continue
      if required_attrs_dict.has_key(a_oid):
        required_attrs.append(a)
      elif allowed_attrs_dict.has_key(a_oid):
        allowed_attrs.append(a)
      else:      
        nomatching_attrs.append(a)

    PrintAttrList(sid,outf,form,ls,dn,sub_schema,display_entry,required_attrs,'Required Attributes')
    PrintAttrList(sid,outf,form,ls,dn,sub_schema,display_entry,allowed_attrs,'Allowed Attributes')
    PrintAttrList(sid,outf,form,ls,dn,sub_schema,display_entry,nomatching_attrs,'Various Attributes')

    # Display operational attributes with template as footer
    if read_output=='template':
      outf.write(operational_attrs_template % display_entry)
    outf.write('</div>\n')

    w2lapp.gui.PrintFooter(outf,form)

  elif read_output=='vcard':

    ##############################################################
    # vCard export
    ##############################################################

    template_dict = ldap.cidict.cidict(
      w2lapp.cnf.GetParam(ls,'%s_template' % read_output,{})
    )
    template_oc = intersection(
      [ s.lower() for s in entry.get('objectClass',[]) ],
      [ s.lower() for s in template_dict.keys() ],
      ignorecase=1
    )

    if template_oc:
      # Templates defined => display the entry with the help of a template
      display_entry = msbase.CaseinsensitiveStringKeyDict(default='')
      display_entry['dn'] = dn.encode('iso-8859-1')
      for attr_type_name in entry.keys():
        itemlist = []
        if w2lapp.schema.no_humanreadable_attr(sub_schema,attr_type_name):
          continue
        try:
          for item in ldaputil.base.unicode_list(entry[attr_type_name],ls.charset):
            itemlist.append('\, '.join(item.encode('iso-8859-1').split('$')))
        except UnicodeError:
          for item in ldaputil.base.unicode_list(entry[attr_type_name],'iso-8859-1'):
            itemlist.append('\, '.join(item.encode('iso-8859-1').split('$')))
	display_entry[attr_type_name] = ';'.join(itemlist)

      oc = template_oc[0]
      vcard_template_filename = w2lapp.gui.GetVariantFilename(template_dict[oc],form.accept_language)
      if vcard_template_filename:
	try:
  	  template_str = open(vcard_template_filename,'r').read()
	except IOError:
	  raise w2lapp.core.ErrorExit(
            """I/O error during reading template file for 
            object class <strong>%s</strong>.""" % (oc)
          )
        else:
          pyweblib.httphelper.SendHeader(
            outf,'text/x-vcard',
            charset='ISO-8559-1',
            expires_offset=w2lapp.cnf.misc.sec_expire,
            additional_header={'Cache-Control':'no-cache'}
          )
  	  outf.write(template_str % display_entry)
      else:
        raise w2lapp.core.ErrorExit(
          """vCard template file for object class
          <strong>%s</strong> not found.""" % (oc)
        )

    else:
      raise w2lapp.core.ErrorExit(
        'No template file found for object class(es) of this entry.'
      )
